#!/usr/bin/python3
"""
Copyright (C) 2025 Alastair McKinstry <mckinstry@debian.org>
Released under the GPL-3 GNU Public License.
"""

# from importlib.metadata import version as __version__

DEPENDS_SUBSTVARS = {
    "fortran": "${fortran:Depends}",
}
PKG_NAME_TPLS = {
    "fortran": ("libfortran-", "fortran."),
}
# options.command = {"fortran": "dh-fortran"}



#!/usr/bin/env python3
"""
Compiler ID  handling for dhfortran: tests

Copyright (C) 2025 Alastair McKinstry <mckinstry@debian.org>
Released under the GPL-3 GNU Public License.

"""

import dhfortran.compilers as cp

def test_get_flavor():
    gf = cp.default_compilers["gfortran"]
    arch = cp.multiarch
    assert cp.get_fc_flavor_arch("/usr/bin/gfortran") == (gf, arch)
    assert cp.get_fc_flavor_arch("/usr/bin/gfortran-15") == (gf, arch)
    assert cp.get_fc_flavor_arch(f"/usr/bin/{arch}-gfortran-15") == (gf, arch)
    assert cp.get_fc_flavor_arch("/etc/alternatives/f95") == (gf,arch)

    try:
        x,_ = cp.get_fc_flavor_arch("/bin/garbage"), arch
    except Exception as ex:
        pass
    else:
        assert 1
        1, "get_fc_flavor_arch('/bin/garbage') should fail"


def test_get_fc_default():
    gf = cp.default_compilers["gfortran"]
    assert cp.get_fc_default() == gf



def test_get_optional():
    assert cp.get_fc_optional() is not None


if __name__ == "__main__":
    import pytest

    pytest.main()



#!/usr/bin/python3
"""
Compiler ID  handling for dhfortran

Copyright (C) 2025-2026 Alastair McKinstry <mckinstry@debian.org>
Released under the GPL-3 GNU Public License.

This module includes all compiler-specific stuff, typically passed to
"""

import os
import functools
from click.exceptions import UsageError
from pathlib import Path
from subprocess import check_output
from shutil import which


multiarch = (
    check_output(["dpkg-architecture", "-qDEB_HOST_MULTIARCH"]).strip().decode("utf-8")
)

# To be updated as gfortran, flang etc are updated  for new versions
default_compilers = {
    "gfortran": "gfortran-16",
    "flang": "flang-22",
    "flangext": "flang-to-external-fc-18",
}


# Compiler information.
# ABI names compatible with CMake Identifiers (lower case)
# first exe name is the default

compilers = {
    "gfortran-7": {
        "exe": ["gfortran-7"],
        "abi": "GNU",
        "mod": "gfortran-mod-14",
        "cc" : "gcc",
        "cxx" : "g++",
        "compatible_moddirs": ["gfortran-mod-14", "flangext-mod-14"],
    },
    "gfortran-13": {
        "exe": ["gfortran-13"],
        "abi": "GNU",
        "mod": "gfortran-mod-15",
        "cc" : "gcc",
        "cxx" : "g++",
        "compatible_moddirs": ["gfortran-mod-14", "flangext-mod-14"],
        "compatible_moddirs": ["gfortran-mod-15", "flangext-mod-15"],
    },
    "gfortran-14": {
        "exe": ["gfortran-14"],
        "abi": "GNU",
        "mod": "gfortran-mod-15",
        "cc" : "gcc",
        "cxx" : "g++",
        "compatible_moddirs": ["gfortran-mod-14", "flangext-mod-14"],
        "compatible_moddirs": ["gfortran-mod-15", "flangext-mod-15"],
    },
    "gfortran-15": {
        "exe": ["gfortran-15"],
        "abi": "GNU",
        "mod": "gfortran-mod-16",
        "cc" : "gcc",
        "cxx" : "g++",
        "compatible_moddirs": ["gfortran-mod-14", "flangext-mod-14"],
        "compatible_moddirs": ["gfortran-mod-16", "flangext-mod-16"],
    },
    "gfortran-16": {
        "exe": ["gfortran-16"],
        "abi": "GNU",
        "mod": "gfortran-mod-16",
        "cc" : "gcc",
        "cxx" : "g++",
        "compatible_moddirs": ["gfortran-mod-14", "flangext-mod-14"],
        "compatible_moddirs": ["gfortran-mod-16", "flangext-mod-16"],
    },
    "flang-new-17": {"exe": ["flang-new-17"], "abi": "Flang", "mod": "flang-mod-1"},
    "flang-new-18": {"exe": ["flang-new-18"], "abi": "Flang", "mod": "flang-mod-1"},
    "flang-new-19": {"exe": ["flang-new-19"], "abi": "flang", "mod": "flang-mod-19"},
    "flang-new-20": {"exe": ["flang-new-20"], "abi": "flang", "mod": "flang-mod-20"},
    "flang-21": {
        "exe": ["flang-new-21", "flang-21"],
        "abi": "flang",
        "mod": "flang-mod-21",
        "cc" : "clang-21",
        "cxx" : "clang++-21",
    },
    "flang-22": {
        "exe": ["flang-new-22", "flang-22"],
        "abi": "flang",
        "mod": "flang-mod-22",
        "cc" : "clang-22",
        "cxx" : "clang++-22",
    },
    "flang-23": {
        "exe": ["flang-new-23", "flang-23"],
        "abi": "flang",
        "mod": "flang-mod-23",
        "cc" : "clang",
        "cxx" : "clang++",
    },
    "lfortran": {"exe": ["lfortran"], "abi": "lcc", "mod": "lfortran-mod-0", "cc" : "clang", "cxx" : "clang++"},
    "flang-to-external-fc-17": {
        "exe": ["flang-to-external-fc-17"],
        "abi": "flangext",
        "mod": "flangext-mod-15",
        "compatible_moddirs": ["flangext-mod-15", "gfortran-mod-15"],
    },  # flangext not in CMake list. Now obsolete anyway
    "flang-to-external-fc-18": {
        "exe": ["flang-to-external-fc-18"],
        "abi": "flangext",
        "mod": "flangext-mod-15",
        "compatible_moddirs": ["flangext-mod-15", "gfortran-mod-15"],
    },
    # Commercial compilers. Need to be checked TODO
    "intel": {
        "exe": ["ifx", "ifc", "efc", "iort"],
        "abi": "Intel",
        "mod": "UNKNOWN",
        "mod_arg": "-module ",
    },
    "pgi": {
        "exe": ["pgfortran", "pgf95", "pgf90", "pgf77"],
        "abi": "pgi",
        "mod": "UNKNOWN",
    },
}

# Compatible ABIs. TODO
compatible = {
    "GNU": ["GNU"],
}


# See  #957692
# TODO: Rework this with  Guillem, a plugin mechanism for dpkg-buildflags

# fPIC over the top and unnecessary in many cases, but workaround for fpm
flags_changes = {
    "gfortran-10": {
        "FCFLAGS": {"append": ["-fallow-invalid-boz", "-fallow-argument-mismatch"]}
    },
    "gfortran-11": {
        "FCFLAGS": {"append": ["-fallow-invalid-boz", "-fallow-argument-mismatch"]}
    },
    "gfortran-12": {
        "FCFLAGS": {"append": ["-fallow-invalid-boz", "-fallow-argument-mismatch"]}
    },
    "gfortran-13": {
        "FCFLAGS": {"append": ["-fallow-invalid-boz", "-fallow-argument-mismatch"]}
    },
    "gfortran-14": {
        "FCFLAGS": {"append": ["-fallow-invalid-boz", "-fallow-argument-mismatch"]}
    },
    "gfortran-15": {
        "FCFLAGS": {"append": ["-fallow-invalid-boz", "-fallow-argument-mismatch"]}
    },
    "gfortran-16": {
        "FCFLAGS": {"append": ["-fallow-invalid-boz", "-fallow-argument-mismatch"]}
    },
    "flang-7": {"FCFLAGS": {"strip": ["-g"]}},
    "flang-new-18": {
        "FCFLAGS": {
            "strip": [
                "-mbranch-protection=standard",
                "-fstack-protector-strong",
                "-fstack-clash-protection",
                "-ffile-prefix-map%",
                "-fcf-protection",
            ]
        },
        "LDFLAGS": {"strip": ["-mbranch-protection=standard"]},
    },
    "flang-new-19": {
        "FCFLAGS": {
            "strip": [
                "-mbranch-protection=standard",
                "-fstack-protector-strong",
                "-fstack-clash-protection",
                "-ffile-prefix-map%",
                "-fcf-protection",
            ]
        },
        "LDFLAGS": {"strip": ["-mbranch-protection=standard"]},
    },
    "flang-new-20": {
        "FCFLAGS": {
            "strip": [
                "-mbranch-protection=standard",
                "-fstack-protector-strong",
                "-fstack-clash-protection",
                "-ffile-prefix-map%",
                "-fcf-protection",
            ]
        },
        "LDFLAGS": {"strip": ["-mbranch-protection=standard"]},
    },
    "flang-21": {
        "FCFLAGS": {
            "append": ["-fPIC"],  # fPIC unnecessary for static, but workaround for fpm
            "strip": [
                "-mbranch-protection=standard",
                "-fstack-protector-strong",
                "-fstack-clash-protection",
                "-ffile-prefix-map%",
                "-fcf-protection",
            ],
        },
        "LDFLAGS": {"strip": ["-mbranch-protection=standard"]},
        "CC": {"replace": ["clang-21"]},
        "CXX": {"replace": ["clang++-21"]},
        "CPPFLAGS": {"strip": ["-Wdate-time"]},
    },
    "flang-22": {
        "FCFLAGS": {
            "append": ["-fPIC"],  # fPIC unnecessary for static, but workaround for fpm
            "strip": [
                "-mbranch-protection=standard",
                "-fstack-protector-strong",
                "-fstack-clash-protection",
                "-ffile-prefix-map%",
                "-fcf-protection",
            ],
        },
        "LDFLAGS": {"strip": ["-mbranch-protection=standard"]},
        "CC": {"replace": ["clang-22"]},
        "CXX": {"replace": ["clang++-22"]},
        "CPPFLAGS": {"strip": ["-Wdate-time"]},
    },
    "flang-23": {
        "FCFLAGS": {
            "append": ["-fPIC"],  # fPIC unnecessary for static, but workaround for fpm
            "strip": [
                "-mbranch-protection=standard",
                "-fstack-protector-strong",
                "-fstack-clash-protection",
                "-ffile-prefix-map%",
                "-fcf-protection",
            ],
        },
        "LDFLAGS": {"strip": ["-mbranch-protection=standard"]},
        "CC": {"replace": ["clang-23"]},
        "CXX": {"replace": ["clang++-23"]},
        "CPPFLAGS": {"strip": ["-Wdate-time"]},
    },
    "lfortran": {
        "FCFLAGS": {
            "strip": [
                "-mbranch-protection=standard",
                "-fstack-protector-strong",
                "-fstack-clash-protection",
                "-ffile-prefix-map%",
                "-fcf-protection",
            ],
            "append" : [ "--cpp" , "--separate-compilation" ] ,
        }
    },
}

def compiler_present(f):
    """Is a given compiler present on the system ? """
    if which(f):
        return True
    if f in default_compilers:
        if which(default_compilers[f]) is not None:
            return True
    return False


@functools.cache
def rework_flags(flavor: str, host_arch: str) -> list[str]:
    """edit env flags.
    TODO: this is the dpkg env, user may have changed
    """
    if flavor not in flags_changes:
        # cli.verbose_print(f"No env flag changes needed for {flavor}")
        return ""
    old_flags = check_output(["dpkg-buildflags"]).strip().decode("utf-8")
    old_env, new_env = {}, {}
    for line in old_flags.split("\n"):
        i = line.index("=")
        key, val = line[:i], line[i + 1 :]
        old_env[key] = val
    for var in flags_changes[flavor]:
        if "replace" in flags_changes[flavor][var]:
            new_env[var] = " ".join(flags_changes[flavor][var]["replace"])
            continue
        flags_list = old_env[var].split()
        if "strip" in flags_changes[flavor][var]:
            for r in flags_changes[flavor][var]["strip"]:
                if "%" in r:
                    prefix = r[: r.index("%")]
                    for x in flags_list:
                        if x.startswith(prefix):
                            flags_list.remove(x)
                else:
                    if r in flags_list:
                        flags_list.remove(r)
        if "append" in flags_changes[flavor][var]:
            flags_list += flags_changes[flavor][var]["append"]
        new_env[var] = " ".join(flags_list)
    return new_env


@functools.cache
def get_fc_flavor_arch(fc=None) -> str:
    """Given a compiler name, work out flavor and host arch
    Works with cross-compile, validates fc or fails with UsageError
    """
    # TODO: mpifort , h5fc ?
    # resolve symlinks and strip prefix for cross-compile , etc
    orig_fc = fc
    arch = multiarch
    if fc is None and "FC" in os.environ:
        f = os.environ["FC"]
    else:
        f = fc if fc else "/etc/alternatives/f95"
    if f in default_compilers:
        f = default_compilers[f]
    fullpath = which(f)
    if fullpath is None:
        fullpath = which(default_compilers['gfortran'])
    if fullpath is None:
        if orig_fc is None:
            orig_fc='gfortran' # for better error message
        raise Exception(f"fc compiler {orig_fc} is broken;"
                        "bad symlink or missing install?")
    fc = Path(fullpath).resolve().name
    m = (
        check_output(["dpkg-architecture", "-qDEB_BUILD_GNU_TYPE"])
        .strip()
        .decode("utf-8")
    )
    if fc.startswith(m):
        fc = fc[len(m) + 1 :]
    # cross-compiling
    m = (
        check_output(["dpkg-architecture", "-qDEB_HOST_GNU_TYPE"])
        .strip()
        .decode("utf-8")
    )
    if fc.startswith(m):
        fc = fc[len(m) + 1 :]
        arch = m
    for f in compilers:
        if fc in compilers[f]["exe"]:
            return f, arch
    raise UsageError(f"Can't recognize compiler {fc}")


@functools.cache
def get_fmoddir(flavor: str, host_arch: str = None) -> str:
    f = flavor if flavor else default_compilers["gfortran"]
    host_arch = multiarch if host_arch is None else host_arch
    flibdir = f"/usr/lib/{host_arch}/fortran"
    modv = compilers[f]["mod"]
    return f"{flibdir}/{modv}"


def get_fmoddirs(flavor: str, host_arch: str) -> str:
    f = flavor if flavor else default_compilers["gfortran"]
    host_arch = multiarch if host_arch is None else host_arch
    flibdir = f"/usr/lib/{host_arch}/fortran"
    if "compatible_moddirs" in compilers[f]:
        return [f"{flibdir}/{modv}" for modv in compilers[f]["compatible_moddirs"]]
    else:
        return [get_fmoddir(f)]


def get_flibdir(flavor: str, host_arch: str) -> str:
    f = flavor if flavor else default_compilers["gfortran"]
    host_arch = multiarch if host_arch is None else host_arch
    flibdir = f"/usr/lib/{host_arch}/fortran"
    abi = compilers[f]["abi"].lower()
    return f"{flibdir}/{abi}"


def get_flibdirs(flavor: str, host_arch: str) -> str:
    f = flavor if flavor else default_compilers["gfortran"]
    host_arch = multiarch if host_arch is None else host_arch
    if "compatible_libdirs" in compilers[f]:
        flibdir = f"/usr/lib/{host_arch}/fortran"
        return [f"{flibdir}/{lib}" for lib in compilers[f]["compatible_libdirs"]]
    else:
        return [get_flibdir(f)]


def get_fc_flags(flavor: str, host_arch: str) -> str:
    flags = rework_flags(flavor, host_arch)
    if "LDFLAGS" in flags:
        ldflags = flags["LDFLAGS"]
    else:
        ldflags = os.environ["LDFLAGS"] if 'LDFLAGS'in os.environ else ""
    fcflags = flags["FCFLAGS"] + " " + ldflags
    fmoddir = get_fmoddir(flavor, host_arch)
    fcflags += f" -I{fmoddir}"

    return fcflags


def get_ld_flags(flavor: str, host_arch: str) -> str:
    """Get the LDFLAGS to use if $FC is used as a linker.
    TODO: dpkg-buildflags presumes GNU linker, breaks when flang used
    TODO: multiarch aware
    TODO: rework in more general format
    """
    flags = rework_flags(flavor, host_arch)
    flibdir = get_flibdir(flavor, host_arch)
    if "LDFLAGS" in flags:
        ldflags = flags["LDFLAGS"]
    else:
        ldflags = os.environ["LDFLAGS"] if 'LDFLAGS'in os.environ else ""
    ldflags += f" -L{flibdir}"

    return ldflags


def get_preferred(preferred: str, flavor: str) -> str:
    """When links are set up and multiple versions of a library can/may be installed.
    which one is installed ? which links are set up?

    """
    if preferred is not None:
        return preferred
    if "PREFERRED" in os.environ:
        return os.environ["PREFERRED"]
    f = flavor if flavor in compilers else get_fc_default()
    return get_fc_default(f)


def get_f77(f77=None):
    if f77 is None and "F77" in os.environ:
        f = os.environ["F77"]
    else:
        f = f77 if f77 else "/etc/alternatives/f77"
    fullpath = which(f)
    if fullpath is None:
        fullpath = which(default_compilers['gfortran'])
    if fullpath is None:
        if f is None:
            f='gfortran' # for better error message
        raise Exception(f"f77 compiler {f} is broken; bad symlink or missing?")
    return Path(fullpath).resolve().name


@functools.cache
def get_fc_default(fc=None) -> str:
    """Return the default Fortran compiler"""

    def canon(f):
        return default_compilers[f] if f in default_compilers else f
    if "FC_DEFAULT" in os.environ:
        return canon(os.environ["FC_DEFAULT"])
    if fc is not None:
        abi = get_abi_vendor(fc)
        if abi in default_compilers:
            d = default_compilers[abi.lower()]
            if which(d) is not None:
                return d
    default, _ = get_fc_flavor_arch(fc)
    if which(default) is None:
        raise Exception(f"Cannot find default compiler {default}")
    return canon(default)


@functools.cache
def get_fc_optional(fc=None) -> set[str]:
    """Return the list of other compilers present on the system
    Remove the default from the list if given"""
    d = get_fc_default(fc)
    if "FC_OPTIONAL" in os.environ:
        flavors = set(f for f in os.environ["FC_OPTIONAL"].split(" "))
        if "" in flavors:
            flavors.remove("")
        if d in flavors:
            flavors.remove(d)
        for f in list(flavors):
            if not compiler_present(f):
                flavors.remove(f)
        return flavors
    else:
        compilers_present = set()
        # NOTE Disable search functionality until dh-fortran is more mature ...
        #  its surprising behaviour and unwanted right now
        #
        # for flavor in compilers:
        #     for exes in compilers[flavor]["exe"]:
        #         if os.path.exists(f"/usr/bin/{exes}"):
        #             if flavor != d and flavor not in compilers_present:
        #                 compilers_present.add(flavor)
        return compilers_present


@functools.cache
def get_fc_required(fc=None) -> set[str]:
    """Return the list of compilers that must work and libraries ship.
    Will include $(FC_DEFAULT), not overlap with $(FC_OPTIONAL)
    TODO: complete, error check
    """
    reqd = set()
    reqd.add(get_fc_default(fc))
    if "FC_REQUIRED" in os.environ:
        for f in os.environ["FC_REQUIRED"].split(" "):
            reqd.add(f)
    return reqd


@functools.cache
def is_multifortran_build():
    r = get_fc_required()
    o = get_fc_optional()
    return len(r) + len(o) > 1


def get_pkg_config_path(flavor: str, arch: str = None) -> str:
    arch = multiarch if arch is None else arch
    flibdir = f"/usr/lib/{arch}/fortran"
    pkg_config_path = (
        os.environ["PKG_CONFIG_PATH"] if "PKG_CONFIG_PATH" in os.environ else ""
    )
    return ":".join([flibdir + "/" + flavor + "/pkgconfig", pkg_config_path])


def get_cmake_path(flavor: str, arch: str = None) -> str:
    arch = multiarch if arch is None else arch
    flibdir = f"/usr/lib/{arch}/fortran"
    cmake_module_path = (
        os.environ["CMAKE_MODULE_PATH"] if "CMAKE_MODULE_PATH" in os.environ else ""
    )
    return ":".join([flibdir + "/" + flavor + "/cmake", cmake_module_path])


def get_env(flavor: str) -> str:
    # Used internally to set env flags for debhelper perl, but also debugging
    fc, host_arch = get_fc_flavor_arch(flavor)
    fc_default = get_fc_default(fc)
    fc_optional = " ".join(get_fc_optional(fc))
    fc_required = " ".join(get_fc_required(fc))
    fc_flags = get_fc_flags(fc, host_arch)
    fc_ldflags = get_ld_flags(fc, host_arch)
    cc = compilers[fc]['cc']
    cxx = compilers[fc]['cxx']
    result = f"""FC={fc}
FC_DEFAULT={fc_default}
FC_OPTIONAL={fc_optional}
FC_REQUIRED={fc_required}
PKG_CONFIG_PATH={get_pkg_config_path(fc)}
CMAKE_MODULE_PATH={get_cmake_path(fc)}
FCFLAGS={fc_flags}
FFLAGS={fc_flags}
LDFLAGS={fc_ldflags}
DESTDIR={os.getcwd()}/debian/tmp-{fc}
FPM_FC={fc}
FPM_FFLAGS={fc_flags}
FPM_LDFLAGS={fc_ldflags}
CC={cc}
CXX={cxx}"""
    new_env = rework_flags(flavor, host_arch)
    for var in new_env:
        if var in {"FCFLAGS", "LDFLAGS", "CC", "CXX"}:
            continue
        result += f"\n{var}={new_env[var]}"
    return result


def fc_target_arch_lfortran(fcflags: str) -> str:
    """Return the target triple for an lfortran compilation
    TODO: Needs testing
    """

    flags = fcflags.split()

    if "-target" in flags:
        try:
            target = flags[flags.index("-target") + 1]
        except Exception:
            raise Exception("Parse error finding -target in lfortran flags list")

        # Translate target archs to GNU triplet
        archs = {
            "aarch64": "aarch64-linux-gnu",  #   - AArch64 (little endian)
            "aarch64_32": None,  #  - AArch64 (little endian ILP32) unsupported
            "aarch64_be": "aarch64_be-linux-gnu",  #  (big endian) unsupported 
            "arm64": "aarch64-linux-gnu",  # ARM64 (little endian)
            "arm64_32": None,  #   - ARM64 (little endian ILP32)
            "x86": "i386-linux-gnu",  #     - 32-bit X86: Pentium-Pro and above
            "x86-64": "x86_64-linux-gnu",  #     - 64-bit X86: EM64T and AMD64
        }
        return archs[target]
    return None


def get_fc_exe(flavor=None) -> str:
    """Return compiler name,  suitable for
    -DCMAKE_Fortran_COMPILER=$(call get_fc_exe,XXX)"""
    # TODO: cross-compilation ? Is this even needed, or will
    # fc as flavor do?

    f = flavor if flavor in compilers else get_fc_default()
    return compilers[f]["exe"][0]


def get_abi_vendor(flavor: str) -> str:
    """Return vendor name.
    Names compatible with CMake
    """

    # names from CMake 3.11 Fortran ID, but lowercase

    if flavor.startswith("gfortran") or flavor.startswith("flangext"):
        return "gnu"
    if flavor.startswith("flang"):
        # flang can mean _Fortran_COMPILER_NAMES_Flang or
        # _Fortran_COMPILER_NAMES_LLVMFlang
        return "flang"
    if flavor.startswith("lfortran"):
        # Not presumed to be ABI-compatible with gfortran, but likely
        return "lcc"
    if flavor.startswith("path"):
        # pathf2003 pathf95 pathf90
        return "PathScale"
    if flavor.startswith("pgf"):
        # pgf95 pgfortran pgf90 pgf77
        return "PGI"
    if flavor.startswith("nvfortran"):
        return "NVHPC"
    if flavor.startswith("nagfor"):
        return "NAG"
    if flavor.startswith("xlf"):
        # set(_Fortran_COMPILER_NAMES_XL        xlf)
        # set(_Fortran_COMPILER_NAMES_VisualAge xlf95 xlf90 xlf)
        return "VisualAge"
    if flavor.startswith("af"):
        # af95 af90 af77
        return "Absoft"
    if flavor in ["ifort", "ifc", "efc", "ifx"]:
        return "Intel"

    raise Exception(f"ABI Vendor unknown for flavor {flavor}")


if __name__ == "__main__":
    import pytest

    pytest.main(["tests/compilers.py"])



#!/usr/bin/python3
"""
Pkgconf file handling for Fortran : tests

Copyright (C) 2025 Alastair McKinstry <mckinstry@debian.org>
Released under the GPL-3 Gnu Public License.
"""

from click.testing import CliRunner
import dhfortran.pkgconf as pk


def test_dh_fortran_pkgconf():
    runner = CliRunner()
    result = runner.invoke(pk.dh_fortran_pkgconf, [])
    assert result.exit_code == 0
    assert result.output == ""


def test_dh_fortran_pkgconf_verbose():
    runner = CliRunner()
    result = runner.invoke(pk.dh_fortran_pkgconf, ["--verbose"])
    assert result.exit_code == 0
    assert result.output.startswith("dh_fortran_pkgconf called with")


def test_dh_fortran_pkgconf_fc():
    runner = CliRunner()
    result = runner.invoke(pk.dh_fortran_pkgconf, ["--fc", "gfortran-15"])
    assert result.exit_code == 0
    assert result.output == ""


if __name__ == "__main__":
    import pytest

    pytest.main()



#!/usr/bin/python3
"""
Test Driver for dh-fortran/cli.py

Copyright (C) 2025 Alastair McKinstry <mckinstry@debian.org>
Released under the GPL-3 Gnu Public License.
"""

import dhfortran.cli as cli


def test_validate_flavor():
    cli.validate_flavor(None)    

def test_validate_fc_none():
    cli.validate_compiler(None)
    
def test_get_env():
    pass



if __name__ == "__main__":
    import pytest

    pytest.main()



#!/usr/bin/python3
"""
Test Driver for dh_fortran_lib

Copyright (C) 2025 Alastair McKinstry <mckinstry@debian.org>
Released under the GPL-3 Gnu Public License.
"""

from click.testing import CliRunner
import dhfortran.lib as lib
from dhfortran.compilers import multiarch
import dhfortran.debhelper as dh
import dhfortran.tests.helpers as helpers
import pytest
import os


@pytest.fixture()
def set_env():
    os.environ["NO_ACT"] = "1"


def test_dh_fortran_lib():
    """Install libs in appropriate directory"""
    # helpers.chdir_to_test('tests/pkg')
    runner = CliRunner()
    result = runner.invoke(dh_fortran_lib, [])
    assert result.exit_code == 0
    # assert result.output == 'Hello Peter!\n'


def test_dh_fortran_lib_files_nopkg():
    """Install libs in appropriate directory"""
    # helpers.chdir_to_test('tests/pkg')
    runner = CliRunner()
    result = runner.invoke(dh_fortran_lib, ["debian/tmp/foo"])
    print (result.output)
    assert result.exit_code == 0
    # assert result.output == 'Hello Peter!\n'



if __name__ == "__main__":
    import pytest

    pytest.main()



#!/usr/bin/python3
"""
Module file handling for Fortran

Copyright (C) 2025 Alastair McKinstry <mckinstry@debian.org>
Released under the GPL-3 Gnu Public License.
"""

import click
import magic
import os
import dhfortran.debhelper as dh
import dhfortran.cli as cli
import dhfortran.compilers as cmplrs
import gzip
from subprocess import check_output
import fnmatch

# cache
flang_modfile_hashes = {}


def guess_flang_version(fc, modfile):
    """ flang now ships with "v1" as a format version, but hardcodes a dep on modules
    via a hash. This essentially makes different compiler versions incompatible.
    Try figuring version from module hashes
    """

    def get_hashes_from_modfile(filename , version = None):
        """ Parse a flang v1 modfile, getting hashes,names """        
        with open(filename,'rt') as f:
            lines = f.readlines()
        hash, hashes = '', {}
        for line in lines:
            match line:
                case l if l.find('!mod$ v1 sum:') != -1:
                    hash = l[l.rfind(':')+1:-1]
                case l if l.startswith('!need$'):
                    _,hash,_,name = l.split(' ')
                    hashes[hash] = (version, name[:-1])
                case l if l.startswith('module'):
                    _,name = l.split(' ')
                    hashes[hash] = (version, name[:-1])
                    return hashes, hash
                case _:
                    # something strange happened
                    return hashes, hash

    if fc is None and 'FC' in os.environ:
        fc = os.environ['FC']
    if  fc is not None:
        if fc.startswith('flang-'):
            mod_version = fc[6:] # got the version
            return mod_version
                
    # Do it the hard way.
    if flang_modfile_hashes == {}:
        # Build db from system modules
        flang_versions = set([ key[-2:] for key in cmplrs.compilers.keys()
                         if key.startswith('flang-')])
        for fv in flang_versions:
            for root,dir,files in os.walk(f"/usr/lib/llvm-{fv}/include/flang"):
                for m in fnmatch.filter(files,"*.mod"):
                    hashes, _ = get_hashes_from_modfile(f"{root}/{m}",fv)
                    flang_modfile_hashes.update(hashes)
                    
    hashes, myhash = get_hashes_from_modfile(modfile)
    if myhash in flang_modfile_hashes:
        myver,_ = flang_modfile_hashes[myhash]
        for h in hashes:
            flang_modfile_hashes[h] = (myver, hashes[h][1])
        return myver
    # failed, just use default
    return cmplrs.default_compilers['flang'][6:]
            

    
def which_compiler_flavor_and_mod_version(fc: str, modfile: str) -> tuple[str, str]:
    """For a modfile (path), return a tuple of strings for the compiler
    and modversion"""

    # Helper .
    def get_first_n_lines(modfile,n=2 ):
        try:
            magick = magic.from_file(modfile)
        except Exception as ex:
            raise Exception(f"Can't open modfile {modfile}: {ex}")
        if magick.startswith("gzip compressed"):
            with gzip.open(modfile, "rt") as f:
                return [f.readline(), f.readline()]
        if "text" in magick:
            with open(modfile, "rt") as f:
                # Strip any Unicode BOMs
                return [f.readline().replace("\ufeff", ""), f.readline()]
        lines = check_output(["strings", modfile]).decode("utf-8").split("\n")
        return lines[0:n]

 
    lines = get_first_n_lines(modfile,2)
    match lines[0]:
        case line if line.startswith("GFORTRAN"):
            mod_version = line.split()[3][1:-1]
            if line.find("created from flang") != -1:
                compiler = "flangext"
            else:
                compiler = "gfortran"
        case line if line.startswith("LCompilers Modfile"):
            compiler, mod_version = "lfortran", "0" # lines[1]
        case line if line.startswith("V35"):
            compiler, mod_version = "nvhpc", "35"
        case line if line.startswith("V"):
            # could also be PGI, but we don't ship PGI on Debian ... its an old flang
            compiler = "flang"
            mod_version = line.split()[1][1:]
        case line if line.startswith("!mod$"):
            compiler = "flang"
            mod_version = guess_flang_version(fc, modfile)
        case line if line.startswith("k"):
            # Intel: k820309, 2021.13.0
            # TODO check if version-specific
            compiler, mod_version = "intel", "0"
        case _:
            raise Exception(f"Can't parse file: {modfile}")

    return (compiler, mod_version)


##
## Modfile-specific interface for debhelper
##
class ModFileHelper(dh.DhFortranHelper):

    def __init__(self, options: dict):
        super().__init__(options, "fortran-mod", "dh_fortran_mod")
        # For tracking which compilers we need
        self.modversions = {}

    def can_work_automatically(self, *cmdline_files) -> bool:
        """Should we automatically automagically detect mod files, 
        or work on specified packages?

        Run if we haven't run already, haven't been passed cmdline files,
        """
        if len(cmdline_files) > 0:
            return False
        for p in self.packages:
            if self.has_acted_on_package(p):
                return False
        if len(self.config_file_pkgs) != 0:
            cli.verbose_print(
                "dh_fortran won't run automatically: has specified fortran-mod files"
            )
            return False
        if len(self.libdevel_pkgs) != 1:
            return False
        pkg = self.libdevel_pkgs[0]
        cli.verbose_print(
            f"No fortran-mod files found,1 devel package {pkg}:"
            "may install mod files automatically"
        )
        return True

    def autogenerate_pkg_filelist(
        self, seachdirs: list[str], dest=None
    ) -> list[tuple[str, str, str]]:
        """Return list of (pkg, file_pattern) tuple to work on.
        We walk the search path looking for files to operate on
        """
        mfiles = []
        for d in seachdirs:
            if d is None:
                continue
            for root, dir, files in os.walk(d):
                for item in fnmatch.filter(files, "*.mod"):
                    mfiles.append(f"{root}/{item}")
                for item in fnmatch.filter(files, "*.smod"):
                    mfiles.append(f"{root}/{item}")

        return [(self.libdevel_pkgs[0], f, dest) for f in mfiles]

    def compute_dest(self, modfile, target_pkg, target_dest=None):
        """Where does modfile go ?"""
        # cli.debug_print("compute_dest [mod]  {modfile} {target_dest}")
        # Should be called by base DebHelper() to move files
        cmplr, version = which_compiler_flavor_and_mod_version(self.options.fc, modfile)
        fmoddir = f"/usr/lib/{cmplrs.multiarch}/fortran/{cmplr}-mod-{version}"
        if target_dest is None:
            return fmoddir
        if target_dest.startswith("/"):
            return target_dest
        else:
            return f"{fmoddir}/{target_dest}"

    def process_file(self, pkg, oldfile, target_pkg, target_dest=None):
        """Called by DebHelper
        Module file gets copied; metadata gets stored"""
        cli.debug_print(
            f"process_file [module]  {oldfile=} {target_pkg=} {target_dest=}"
        )
        try:
            compiler, version = which_compiler_flavor_and_mod_version(self.options.fc,oldfile)
        except Exception as ex:
            cli.verbose_print(f"skipping unknown file {oldfile}")
            return
        if pkg not in self.modversions:
            self.modversions[pkg] = set()
        self.modversions[pkg].add(f"{compiler}-mod-{version}")
        dest = self.compute_dest(oldfile, target_pkg, target_dest)
        assert target_pkg != '' and target_pkg is not None
        dh.install_dir(f"debian/{target_pkg}/{dest}")
        os.chmod(oldfile, 0o644)
        self.doit(["cp", oldfile, f"debian/{target_pkg}/{dest}"])
        #if oldfile.startswith(self.options.tmpdir):
        #    self.doit(["rm", oldfile])


@click.command(
    context_settings=dict(
        ignore_unknown_options=False,
    )
)
@click.option("--fc", help="Fortran compiler flavor (eg gfortran-14, flang-21)")
@click.argument("files", nargs=-1, type=click.UNPROCESSED)
@cli.debhelper_common_args
def dh_fortran_mod(files, *args, **kwargs):
    """
    B<dh_fortran_mod> is a debhelper program that finds Fortran module 
    and submodule files and adds dependencies to B<gfortran-$version> 
    as required to the package using via the variables
    B<$fortran:Depends} and B<${fortran:Recommends}>.

    B<dh_fortran_mod> is expected to be automatically added using 
    the debhelper "addon" B<fortran_mod> ie. either automatically, 
    by build-depending on 'dh-sequence-fortran', or explicitly:

        dh $@ --with fortran


    B<dh_fortran_mod>Searches the debian/ directory for files 
    B<debian/pkg.fortran-mod>$ which list module files to include, 
    with the same syntax as debhelper install files.

    =head1 OPTIONS

    =over 4

    =item B<--tmpdir=>I<dir>

    Look in the specified directory for files to be installed (by default debian/tmp)

    Typically Fortran module files are included in library development packages.

    =back

    =head1 TODO

    Add dh-fortran-mod support for generic fortran compilers (ifx, etc).

    =over 4

    B<dh_fortran_mod> will be expanded to find mod files automatically from the 
    I<debian/tmp> directory.
    It will enable the installation of mod files in parallel for multiple compilers.
    It will install .smod files for Fortran 2018.

    The fortran-mod file syntax follows dh_install: pairs of sources and optional
    target directories.  The default directory will be $fmoddir 
    ( /usr/lib/$multiarch/fortran/$compiler_mod_directory/)
    If the target directory is absolute (starts with a  '/'), this directory is used
    in the target package.  If the target  does not absolute, it will be treated 
    as a subdirectory of $fmoddir.

    Currently four flavours of Fortran compiler are supported: gfortran-*  ('gfortran'),
    flang-new-* ('flang') , flang-to-external-fc-* ('flangext') and 
    lfortran ('lfortran').

    $compiler_mod_directory is based on the compiler module version: 
    currently gfortran-mod-16 for gfortran-15 (and older), flang-mod-1
    for flang-new-18+ and lfortran-mod-0 for lfortran.

    For flang-to-external-fc-* the version is flangext-mod-15 (assuming 
    gfortran-15 as the external compiler); in principle flang-to-external-fc 
    ('flangext' flavour) and gfortran are intercompatible but intermixing is avoided.

    These will be updated for incompatible compiler versions.
        
    """
    cli.debug_print(f"dh_fortran_mod called with files {files} kwargs {kwargs}")

    d = ModFileHelper(dh.build_options(**kwargs))

    searchdirs = [d.options.tmpdir, "."]
    d.process_and_move_files(searchdirs, *files)
    depends = []
    recommends = []
    fc_default = cmplrs.get_fc_default()
    fc_optional = cmplrs.get_fc_optional()
    for package in d.packages:
        if package in d.modversions:
            for m in sorted(d.modversions[package]):
                default_comps = set()
                optional_comps = set()
                for c in cmplrs.compilers:
                    if cmplrs.compilers[c]["mod"] == m:
                        if c  ==   fc_default:
                            default_comps.add(c)
                        if c in fc_optional:
                            optional_comps.add(c)
                if r:= "| ".join(sorted(default_comps)):
                    depends.append(f"{r} | {m}")
                if r:= "| ".join(sorted(optional_comps)):
                    recommends.append(f"{r} | {m}")
            d.addsubstvar(package, "fortran:Depends", ",".join(depends))
            d.addsubstvar(package, "fortran:Recommends", ",".join(recommends))
        if "config_file" in d.packages[package] and package in d.modversions:
            # config file defined, so create cleanup script
            for modversion in d.modversions[package]:
                d.autoscript(
                    package,
                    "postrm",
                    "postrm-fortran-mod",
                    {
                        "MULTIARCH": cmplrs.multiarch,
                        "FCOMPILERMOD": modversion,
                    },
                )
    d.save_substvars()


if __name__ == "__main__":
    import pytest

    pytest.main(["tests/module.py"])



#!/usr/bin/python3
"""
Support routines for dh-fortran cli commands

Copyright (C) 2025 Alastair McKinstry <mckinstry@debian.org>
Released under the GPL-3 Gnu Public License.
"""

import dhfortran.compilers as cmplrs
import traceback
import click
import os
import sys
import subprocess
import logging
import json

log = logging.getLogger("dhfortran")

verbose = None
debug = False


def is_multifortran_build():
    d = cmplrs.get_fc_required()
    o = cmplrs.get_fc_optional()
    return len(d) + len(o) > 1


def verbose_print(s: str):
    log.debug(s)
    global verbose
    if verbose is None:
        verbose = os.environ['DH_VERBOSE'] if 'DH_VERBOSE' in os.environ else False
    if verbose:
        click.echo(s)


def debug_print(s: str):
    log.debug(s)
    global debug
    if debug:
        click.echo(s)


def warning(s: str):
    log.warning(s)
    click.echo(s, err=True)


def update_env(kwargs):
    """Helper: on intercept, ensure present and set:
    buildsystem : 'active' buildsytem to use
    tmpdir : to install into.
    builddir : builddir. to build into.
    """
    try:
        j = subprocess.check_output(["dh_assistant", "which-build-system"])
    except subprocess.CalledProcessError:
        print("Error: must be run inside a package source directory")
        sys.exit(1)

    d = json.loads(j.decode("utf-8"))

    if "builddir" not in kwargs or kwargs["builddir"] is None:
        kwargs["builddir"] = d["buildpath"]
    if kwargs["builddir"] == ".":
        if cmplrs.is_multifortran_build():
            kwargs["builddir"] = "debian/build"
    # TODO needed?
    # kwargs['o__builddirectory'] = kwargs['builddir']

    # TODO: Handle the case of '--var','value' as well ?
    if "tmpdir" not in kwargs:
        for arg in sys.argv:
            if arg.startswith("-O--tmpdir="):
                kwargs["tmpdir"] = arg[11:]
    if "tmpdir" not in kwargs:
        kwargs["tmpdir"] = d["dest-directory"]
    if kwargs["tmpdir"] is None:
        kwargs["tmpdir"] = "debian/tmp"

    # Note: 'buildystem' is distinct from 'o__buildsystem'
    # 'buildsystem' comes from --with= (eg. fortran, ecbuild)
    # 'o__buildsystem' may be eg meson+ninja, cmake+makefile
    # TODO: test dropping setting of o__buildsystem
    kwargs["o__buildsystem"] = d["build-system"]
    if "buildsystem" not in kwargs:
        for arg in sys.argv:
            if arg.startswith("-O--buildsystem="):
                kwargs["buildsystem"] = arg[16:]
    if "buildsystem" not in kwargs or kwargs["buildsystem"] is None:
        kwargs["buildsystem"] = kwargs["o__buildsystem"]
    # Replace 'ecbuild' with underlying system
    if kwargs["buildsystem"] == "ecbuild":
        kwargs["buildsystem"] = kwargs["o__buildsystem"]

    assert kwargs["buildsystem"] is not None
    assert "tmpdir" in kwargs
    assert "builddir" in kwargs

    return kwargs


# decorator
def debhelper_common_args(f, minimal=False):
    """This is a wrapper that is the the top-level entry point to dh-fortran commands.
    Validation of arguments, env variables happens here
    """

    # Wrapper to add debhelper common args to all targets
    @click.option(
        "--verbose",
        "-v",
        help="show commands that modify the package build directory",
        is_flag=True,
    )
    @click.option(
        "--tmpdir",
        "-P",
        help="Use  tmpdir for package build staging directory",
    )
    @click.option("--destdir", help="Use destdir as the destination for final install")
    @click.option(
        "--sourcedir",
        "--sourcedirectory",
        "-D",
        envvar="SOURCEDIR",
        default=".",
        hidden=True,
    )
    @click.option("--builddir", "--builddirectory", "-B", help="Build directory")
    # Ignore the following parameters
    @click.option("-O--no-parallel", is_flag=True, hidden=True)
    @click.option("-O--buildsystem", "--buildsystem", "-S", hidden=True)
    @click.option("-O--builddirectory", "-O--builddir", "-O", hidden=True)
    def wrapper(*args, **kwargs):

        global verbose, debug
        if "DH_VERBOSE" in os.environ:
            verbose = os.environ["DH_VERBOSE"]
        else:
            verbose = kwargs["verbose"] if "verbose" in kwargs else False
        if "DH_DEBUG" in os.environ and os.environ["DH_DEBUG"]:
            debug = True
            verbose = True

        kwargs = update_env(kwargs)

        try:
            f(*args, **kwargs)
        except Exception as ex:
            if verbose:
                print(traceback.format_exc())
            raise click.UsageError(ex)

    if not minimal:
        # Options that don't apply to dh_fortran helpers, just dh_fortran_*
        wrapper = click.option(
            "--no-act", help="Don't act, just show the output", is_flag=True
        )(wrapper)
        wrapper = click.option("--package", "-p", help="Act on named package")(wrapper)
        wrapper = click.option("--firstpackage", envvar="FIRSTPACKAGE", hidden=True)(
            wrapper
        )
        wrapper = click.option(
            "--indep",
            "-i",
            help="Act on all arch-independent packages",
            is_flag=True,
        )(wrapper)
        wrapper = click.option(
            "--arch", "-a", help="Act on all arch-dependent packages", is_flag=True
        )(wrapper)
        wrapper = click.option(
            "--remaining-packages",
            help="Act on remaining packages only",
            hidden=True,
        )(wrapper)
        wrapper = click.option(
            "--no-package",
            "-N",
            help="Do  not act on the specified package",
            hidden=True,
        )(wrapper)
        wrapper = click.option(
            "--mainpackage", help="Deprecated and unused", hidden=True
        )(wrapper)
        wrapper = click.option("--create-in-sourcedir", hidden=True)(wrapper)
        wrapper = click.option(
            "--exclude_pkgs",
            "-X",
            help="Exclude certain packages",
            envvar="EXCLUDE_FIND",
            hidden=True,
        )(wrapper)

    return wrapper


def validate_flavor(flavor: str) -> None:
    # Validation moved get_fc_flavor_arch does checking
    try:
        flavor, arch = cmplrs.get_fc_flavor_arch(flavor)
    except Exception as ex:
        raise click.UsageError(ex)


def validate_compiler(fc: str) -> None:
    # Validation moved; get_fc_flavor does checking
    try:
        flavor, arch = cmplrs.get_fc_flavor_arch(fc)
    except Exception as ex:
        raise click.UsageError(ex)


if __name__ == "__main__":
    import pytest

    pytest.main(["tests/cli.py"])



#!/usr/bin/env python3
"""

Helpers for tests
"""


from pathlib import Path
from functools import wraps
import pytest
import os


# @contextmanager
def compiler_present(*args, **kwds):
    """Execute a given block if the named compiler is present, else skip"""
    print(f"compiler_present: args are {args}")


@pytest.fixture
def dry_run():
    os.environ["NO_ACT"] = "1"


def chdir_to_test(testdir: str):
    """Get the current folder of the test"""
    p = Path(__file__).parent.parent / testdir
    pytest.MonkeyPatch.chdir(p)


def make_testdir(dirname: str):
    pass



#!/usr/bin/python3
"""
Test Driver for dh-fortran/main.py

Copyright (C) 2025-2026 Alastair McKinstry <mckinstry@debian.org>
Released under the GPL-3 Gnu Public License.
"""

from click.testing import CliRunner
import dhfortran.compilers as cmplrs
import dhfortran.debhelper as dh


def test_rework_flang21():
    out = cmplrs.rework_flags('flang-21',None)
    breakpoint()
    print(out)

def test_fcflags_flang21():
    out = cmplrs.get_fc_flags('flang-21',None)
    print(out)



#!/usr/bin/python3
"""
Entry point for dh-fortran command

Copyright (C) 2025 Alastair McKinstry <mckinstry@debian.org>
Released under the GPL-3 Gnu Public License.
"""

import click
import dhfortran.compilers as cmplrs
import dhfortran.cli as cli
import dhfortran.targets as targets


@click.group()
@click.pass_context
def dh_fortran(ctx):
    # breakpoint()    # ensure that ctx.obj exists and is a dict
    # (in case `cli()` is called by means other than the `if` block below)
    ctx.ensure_object(dict)


@dh_fortran.command("get_fc_flags")
@click.option("--fc", default=None, help="Fortran compiler")
# @cli.debhelper_common_args
def get_fc_flags(fc):
    """
    Returns set of flags to be set , either passed as FCFLAGS to another
    """
    if fc is None:
        fc = cmplrs.get_fc_default()
    cli.validate_compiler(fc)
    flavor,arch = cmplrs.get_fc_flavor_arch(fc)
    click.echo(cmplrs.get_fc_flags(flavor,arch))


@dh_fortran.command("get_f77_flags")
@click.option("--f77", default=None, help="Fortran compiler")
def get_f77_flags(f77):
    """
    Returns flags for a FORTRAN 77 compiler (recommended contents of FFLAGS)
    """
    cli.validate_compiler(f77)
    click.echo(cmplrs.get_f77_flags(f77))


@dh_fortran.command("get_ld_flags")
@click.option("--fc", default=None, help="Fortran compiler")
# @cli.debhelper_common_args
def get_ld_flags(fc):
    """
    Returns set of flags to be set , either passed as FCFLAGS to another
    """
    cli.validate_compiler(fc)
    flavor, arch = cmplrs.get_fc_flavor_arch(fc)
    click.echo(cmplrs.get_ld_flags(flavor, arch))


@dh_fortran.command("get_fc_default", help="Returns default Fortran flavor")
@click.option("--fc", default=None, help="Fortran compiler")
@click.pass_context
def get_fc_default(ctx, fc):
    """
    Details on get_fc_default
    """
    cli.validate_compiler(fc)
    click.echo(cmplrs.get_fc_default(fc))


@dh_fortran.command("get_fc_optional", help="Returns list of optional Fortran flavors")
@click.option("--fc", default=None, help="Fortran compiler")
@click.pass_context
def get_fc_optional(ctx, fc):
    cli.validate_compiler(fc)
    flavor, arch = cmplrs.get_fc_flavor_arch(fc)
    click.echo(" ".join(cmplrs.get_fc_optional(flavor)))


@dh_fortran.command("get_flibdir", help="Returns flibdir for a given compiler")
@click.pass_context
@click.option("--fc", default=None, help="Fortran compiler (eg gfortran-15, flang-21")
@click.option(
    "--prepend",
    "-L",
    help="Prepend -L to directories as needed",
    is_flag=True,
    default=False,
)
def get_flibdir(ctx, fc, prepend):
    cli.validate_compiler(fc)
    flavor, arch = cmplrs.get_fc_flavor_arch(fc)
    prefix = "-L" if prepend else ""
    click.echo(f"{prefix}{cmplrs.get_flibdir(flavor,arch)}")


@dh_fortran.command(
    "get_flibdirs", help="Returns compatible flibdirs for a given compiler"
)
@click.pass_context
@click.option("--fc", default=None, help="Fortran compiler (eg gfortran-15, flang-21")
@click.option(
    "--prepend",
    "-L",
    help="Prepend -L to directories as needed",
    is_flag=True,
    default=False,
)
def get_flibdirs(ctx, fc, prepend):
    cli.validate_compiler(fc)
    flavor, arch = cmplrs.get_fc_flavor_arch(fc)
    libdirs = cmplrs.get_flibdirs(flavor, arch)
    prefix = "-L" if prepend else ""
    click.echo(" ".join([f"{prefix}{fl}" for fl in libdirs]))


@dh_fortran.command("get_fmoddir", help="Returns fmoddir for a given compiler")
@click.option("--fc", default=None, help="Fortran compiler")
@click.option("--flavor", help="Fortran flavor (eg gfortran-15, flang-21)")
#              deprecate="--flavor is deprecated in favour of --fc")
@click.option(
    "--prepend",
    "-I",
    help="Prepend -I or -M to directories as needed",
    is_flag=True,
    default=False,
)
@click.pass_context
def get_fmoddir(ctx, flavor, fc, prepend):
    cli.validate_compiler(fc)
    flavor, arch = cmplrs.get_fc_flavor_arch(flavor if fc is None else fc)
    # May be "-I" or "-module " depending on compiler
    prefix = ""
    if prepend:
        prefix = (
            cmplrs.compilers[flavor]["mod_arg"]
            if "mod_arg" in cmplrs.compilers[flavor]
            else "-I"
        )
    click.echo(f"{prefix}{cmplrs.get_fmoddir(flavor,arch)}")


@dh_fortran.command(
    "get_fmoddirs", help="Returns compatible fmoddirs for a given compiler"
)
@click.option("--fc", help="Fortran compiler (eg gfortran-15, flang-21)")
@click.option(
    "--prepend",
    "-I",
    help="Prepend -I or -M to directories as needed",
    is_flag=True,
    default=False,
)
@click.pass_context
def get_fmoddirs(ctx, fc, prepend):
    cli.validate_compiler(fc)
    flavor, arch = cmplrs.get_fc_flavor_arch(fc)
    # TODO. May be "-I" or "-M" depending on compiler
    moddirs = cmplrs.get_fmoddirs(flavor, arch)
    prefix = ""
    if prepend:
        prefix = (
            cmplrs.compilers[flavor]["mod_arg"]
            if "mod_arg" in cmplrs.compilers[flavor]
            else "-I"
        )
    click.echo(" ".join([f"{prefix}{moddir}" for moddir in moddirs]))


@dh_fortran.command(
    "get_fc_exe",
    help="Return Fortran executable for specified (or default) flavor",
    deprecated="MAY BE  DEPRECATED IN FINAL API",
)
@click.option("--flavor", help="Fortran flavor (eg gfortran-15, flang-21)")
@click.pass_context
def get_fc_exe(ctx, flavor):
    cli.validate_flavor(flavor)
    click.echo(cmplrs.get_fc_exe(flavor))


@dh_fortran.command("get_abi_vendor", help="Returns ABI name for a given compiler")
@click.option("--fc", help="Fortran compiler  (eg gfortran-15, flang-21)")
@click.pass_context
def get_abi_vendor(ctx, fc):
    cli.validate_compiler(fc)
    flavor, arch = cmplrs.get_fc_flavor_arch(fc)
    click.echo(cmplrs.get_abi_vendor(flavor))


@dh_fortran.command(
    "get_env",
    help="Return set of environment flags to execute  debian/rules targets with",
)
@click.option("--fc", default=None, help="Fortran compiler")
@click.option("--save", default=False, is_flag=True, hidden=True)
def get_env(fc: str, save: bool):
    cli.validate_compiler(fc)
    if fc and not cmplrs.compiler_present(fc):
        raise click.UsageError(Exception(f"Compiler {fc} not on PATH; is it installed?"))

    flavor, arch = cmplrs.get_fc_flavor_arch(fc)
    if save:
        with open(".debhelper/fc_env", "w") as f:
            print(cmplrs.get_env(flavor), file=f)
    else:
        click.echo(cmplrs.get_env(flavor))


##
## debian/rules targets
##


@cli.debhelper_common_args
@dh_fortran.command("clean", help="Execute Fortran-specific clean actions")
def clean():
    targets.clean()


@cli.debhelper_common_args
@dh_fortran.command(
    "configure", help="Execute configure actions for each Fortran flavor"
)
def configure(*args, **kwargs):
    targets.configure()


# Test handling of common debhelper stuff
@cli.debhelper_common_args
@dh_fortran.command("build", help="Execute build target for each Fortran flavor")
def build():
    targets.build()


@cli.debhelper_common_args
@dh_fortran.command("install", help="Execute install target for each Fortran flavor")
def install():
    targets.install()


@cli.debhelper_common_args
@dh_fortran.command("test", help="Execute test target for each Fortran flavor")
def test():
    targets.test()


if __name__ == "__main__":
    import pytest

    pytest.main(["tests/main.py"])
